#include <QtGui>
#include <QtNetwork>
#include <es_sock.h>
#include <in_sock.h>
#include "private/qcore_symbian_p.h"

class MyWidget : public QWidget {
    Q_OBJECT
public:
    MyWidget(QWidget *parent = 0);
    ~MyWidget();
public slots:
    void resolveHost();
private:
    QTextBrowser *browser;
    RSocketServ ss;
    RHostResolver res;
    
    int numResolving; // Added
    QTime start; // Added
};

MyWidget::MyWidget(QWidget *parent) : QWidget(parent)
{
    browser = new QTextBrowser(this);
    QPushButton *button = new QPushButton( tr("Resolve host"), this);
    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->addWidget(browser);
    layout->addWidget(button);
    setLayout(layout);

    QObject::connect(button, SIGNAL(clicked()), this, SLOT(resolveHost()));

    browser->append(QString("App Starting:"));
    TInt err1 = ss.Connect();
    browser->append(QString("   RSocketServ::Connect = %1").arg(err1));
    TInt err2 = res.Open(ss, KAfInet, KProtocolInetTcp);
    browser->append(QString("   RHostResolver::Open = %1").arg(err2));
    
    // Added:
    numResolving = 1;
}

MyWidget::~MyWidget()
{
}

void MyWidget::resolveHost()
{
	// Added
	if (numResolving == 1)
	{
	    start = QTime::currentTime();
	}
	
	// Changed host name: there is actually a problem when resolving Google url
    _LIT(KHostName, "www.google.com");
    
    browser->append(QString("Resolving number: %1").arg(numResolving++));

    // Qt code
    QHostInfo hostInfo = QHostInfo::fromName(qt_TDesC2QString(KHostName));
    browser->append(qt_TDesC2QString(KHostName));
    browser->append(QString("Qt results:"));
    browser->append(QString("   fromName returned: %1").arg(hostInfo.error()));
    if (hostInfo.error() != QHostInfo::NoError)
        browser->append(QString("   errorString: %1").arg(hostInfo.errorString()));

    for (int i = 0; i < hostInfo.addresses().count(); ++i)
        browser->append(QString("   result is %1").arg(hostInfo.addresses().at(i).toString()));
    
    // Symbian code
    TNameEntry entry;
    TInt err;
    err = res.GetByName(KHostName, entry);

    browser->append(QString("Symbian results:"));
    browser->append(QString("   GetByName returned: %1").arg(err));

    // Get the IP address of the host
    TInetAddr hostAddr = entry().iAddr;

    TBuf<50> ipAddr;
    TInetAddr::Cast(hostAddr).Output(ipAddr);

    browser->append(QString("   hostAddr = %1").arg(hostAddr.Address()));
    browser->append(QString("   hostAddr family = %1").arg(hostAddr.Family()));
    QString tmp = QString::fromUtf16(ipAddr.Ptr(), ipAddr.Length());
    browser->append(QString("   result is %1").arg(tmp));
    
    browser->append(QString("Total time spent: %1 secs").arg(start.secsTo(QTime::currentTime())));

    // Added:
    // Stop only on Qt error since for now we never see an error with Symbian code
    if (hostInfo.error() != QHostInfo::HostNotFound)
    {
        // Do not call directly resolveHost() else crashing after a while
	    QMetaObject::invokeMethod(this, "resolveHost", Qt::QueuedConnection);
    }
    else
    {
    	// Reinit numResolving
    	numResolving = 1;
    }
}


int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    MyWidget w;
    w.showMaximized();
    return a.exec();
}

#include "main.moc"
